# frozen_string_literal: true

module Invoca
  module Metrics
    module Prometheus
      module DeclareMetrics
        class Base

          attr_reader :name, :type, :source, :metric, :default_labels, :graphite

          def initialize(name, type, source, metric, labels:, graphite:)
            @name           = name
            @type           = type
            @source         = source
            @metric         = metric
            @default_labels = labels
            @graphite       = graphite

            if labels
              labels.is_a?(Hash) or raise ArgumentError, "Labels must be type Hash but got #{labels.inspect}"
            end
          end

          # TODO: Could default_labels be renamed to labels?
          def settings
            { type: type, labels: default_labels, graphite: graphite }
          end

          # TODO: This is temporary; the invoca-metrics metrics objects
          # should be made immutable, in which case the entire object
          # would be replaced rather than resetting the Prometheus metric
          # object as is done here
          def reset_metric(new_metric)
            @metric = new_metric
          end

          private

          def render_graphite_string(**labels)
            rendered_string = graphite.dup
            labels.map do |key, value|
              rendered_string.gsub!(":#{key}", value.to_s)
            end

            rendered_string.include?(':') and warn("Graphite string #{rendered_string} not fully rendered. Expecting additional label that has not been rendered.")
            rendered_string
          end
        end
      end
    end
  end
end
