# frozen_string_literal: true

require_relative '../configuration'
require_relative 'counter'
require_relative 'gauge'
require_relative 'histogram'

module Invoca
  module Metrics
    module Prometheus
      module DeclareMetrics
        class Dsl

          attr_reader :declared_metrics, :source, :prometheus_exporter_config

          def initialize(source)
            @source = source
            @declared_metrics = []
            if Invoca::Metrics::Prometheus.config_present?
              @prometheus_exporter_config = Invoca::Metrics::Prometheus.config
            end
          end

          def counter(name, labels: {}, graphite: nil, help: nil)
            # TODO: We can drop the 'to_s' when we stop supporting ruby 2.6
            if !name.to_s.end_with?("_total")
              warn("#{source} counter #{name.inspect} should end in \"_total\" so that Grafana can tell that it is a counter.")
            end
            metric =
              if Invoca::Metrics::Prometheus.config_present?
                prometheus_exporter_config.register_metric(:counter, name, help, { labels: labels, graphite: graphite })
              end
            add_metric(Counter.new(name, :counter, source, metric, labels: labels, graphite: graphite))
          end

          def histogram(name, buckets:, labels: {}, graphite: nil, help: nil)
            metric =
              if Invoca::Metrics::Prometheus.config_present?
                prometheus_exporter_config.register_metric(:histogram, name, help, { labels: labels, graphite: graphite, buckets: buckets })
              end
            add_metric(Histogram.new(name, :histogram, source, metric, buckets: buckets, labels: labels, graphite: graphite))
          end

          def gauge(name, labels: {}, graphite: nil, help: nil)
            metric =
              if Invoca::Metrics::Prometheus.config_present?
                prometheus_exporter_config.register_metric(:gauge, name, help, { labels: labels, graphite: graphite })
              end
            add_metric(Gauge.new(name, :gauge, source, metric, labels: labels, graphite: graphite))
          end

          private

          def add_metric(metric)
            @declared_metrics << metric
          end
        end
      end
    end
  end
end
