# frozen_string_literal: true

require_relative 'base'
require 'active_support/deprecation'

module Invoca
  module Metrics
    module Prometheus
      module DeclareMetrics
        class Gauge < Base

          def set(value = :'1', **labels)
            if value == :'1'
              value = 1
              ActiveSupport::Deprecation.warn("gauge default value of 1 is deprecated; please pass an explicit value")
            end

            if metric.present?
              metric.observe(value, default_labels.merge(labels))
            else
              warn("Gauge being set without metric being present")
            end

            if graphite
              Invoca::Metrics::Client.metrics.gauge(render_graphite_string(**default_labels.merge(labels)), value)
            end
          end
        end
      end
    end
  end
end
