# frozen_string_literal: true

module Invoca
  module Metrics
    module Prometheus
      class MetricsRegistry
        class ContradictoryMetricRegistration < StandardError; end

        attr_reader :metrics

        def initialize
          @metrics = {}
        end

        def register(metric)
          validate_metric_name!(metric)
          @metrics[metric.name] = metric
        end

        def method_missing(method_name, *args)
          metrics[method_name] || super
        end

        def respond_to?(method_name, *args)
          @metrics[method_name] || super
        end

        def respond_to_missing?(method_name, include_private = false)
          @metrics[method_name] || super
        end

        private

        def validate_metric_name!(metric)
          if (existing = @metrics[metric.name])
            metric.settings == existing.settings or
              raise ContradictoryMetricRegistration, "metric #{metric.name} already registered by #{existing.source} with contradictory settings #{existing.settings} vs new registered #{metric.source} with setting #{metric.settings}"
          end
        end
      end
    end
  end
end
